/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 *
 *****************************************************************************/
/**
 * @file ZeroizeUtils.cpp
 * Implementation file for CZeroizeUtils class which provides zeroization utilities.
 */


///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////

#include "ZeroizeUtils.h"
#include "ApexException.h"
#include <time.h>
#include <stdlib.h>

#if defined UNDER_CE
#include <windows.h>
#endif


///////////////////////////////////////////////////////////////////////////////
// namespaces
///////////////////////////////////////////////////////////////////////////////

using namespace std;

/**
* Contructor
*/
CZeroizeUtils::CZeroizeUtils()
{

}


/** 
* Destructor
*/
CZeroizeUtils::~CZeroizeUtils()
{

}


/**
* This will zeroize a range of bytes.
*
* @param pcInputBytes - this is the pointer to the bytes to zeroize.
* @param nInputSize - this is the nuber of bytes to zeroize in the pointer.
*/
void CZeroizeUtils::Zeroize(unsigned char* pcInputBytes, size_t nInputSize)
{
	size_t nPassMax = 3;
	size_t nPass = 0;
	size_t i = 0;
	size_t nMatchCount = 0;
	unsigned char c = (unsigned char)0x01;
	size_t nMatchCountMax = (int)(nInputSize/2);
	size_t nRandomCountErrors = 0;
	size_t nRandomCountErrorsMax = 4;

	if (nInputSize%2 > 0)
	{
		nMatchCountMax++;
	}

	// loop three times
	// set value to 0x00
	// set value to 0xFF
	// set value to random
	for (nPass = 0; nPass < nPassMax; nPass++)
	{
		switch (nPass)
		{
			case 0:
				// Set the memory to 0x00
				memset(pcInputBytes, 0x00, nInputSize);

				// Make sure that the memory actually got set.
				for (i =0; i < nInputSize; i++)
				{
					// Get the next character from the buffer.
					c = pcInputBytes[i];

					// Is it set to 0x00?
					if (c != (unsigned char)0x00)
					{
						// If not, then we need to raise an exception.
						wstring strError = L"Error setting the memory to 0x00 in  CZeroizeUtils::Zeroize().";

						throw CApexException( ERROR_FATAL, strError, __FILE__, __LINE__);
					}
				}

				break;

			case 1:
				// Set the memory to 0xFF.
				memset(pcInputBytes, 0xFF, nInputSize);

				// Make sure that the memory actually got set.
				for (i =0; i < nInputSize; i++)
				{
					// Get the next character from the buffer.
					c = pcInputBytes[i];

					// Is it set to 0xFF?
					if (c != (unsigned char)0xFF)
					{
						// If not, then we need to raise an exception.
						wstring strError = L"Error setting the memory to 0xFF in  CZeroizeUtils::Zeroize().";

						throw CApexException( ERROR_FATAL, strError, __FILE__, __LINE__);
					}
				}

				break;

			case 2:
#if defined(UNDER_CE) && defined(_DEBUG)
				// Seed the random number generator.
				srand( (unsigned)GetTickCount() );
#else
				// Seed the random number generator.
				srand( (unsigned)time( NULL ) );
#endif

				// Fill the buffer with random characters.
				for (i = 0; i < nInputSize; i++)
				{
					int nRandom = rand()%255;

					char cRandom = (char)nRandom;

					// Copy the random character to the next location in the buffer.
					memcpy(&pcInputBytes[i], &cRandom, sizeof(char));

					// We want to make sure that we do not place the same character
					// in two adjacent locations in the buffer. Are we past the first
					// buffer location?
					if (i > 0)
					{
						// If so, is the last character written equal to the current?
						if (pcInputBytes[i-1] == pcInputBytes[i])
						{
							// If so, increment our error count.
							nRandomCountErrors++;

							// Are we at the max error count allowed?
							if (nRandomCountErrors == nRandomCountErrorsMax)
							{
								// If so, then we need to raise an exception.
								wstring strError = L"Error providing random memory characters in CZeroizeUtils::Zeroize().";

								throw CApexException( ERROR_FATAL, strError, __FILE__, __LINE__);
							}
						}
						else
						{
							// Reset our error count. We only allow 4 attempts but
							// if we are successful in our write then we want to reset
							// the error count.
							nRandomCountErrors = 0;
						}
					}
				}

				// Check that the memory was really written
				for (i = 0; i < nInputSize; i++)
				{
					// Get the next character from the buffer.
					c = pcInputBytes[i];

					// Is the character equal to 0xFF?
					if (c == (unsigned char)0xFF)
					{
						// If so, increment our match count.
						nMatchCount++;
					}
				}

				// Are more than half the characters in the buffer
				// equal to 0xFF?
				if (nMatchCount > nMatchCountMax)
				{
					// If so, we need to raise and exception.
					wstring strError = L"Error setting the memory to random characters in  CZeroizeUtils::Zeroize().";

					throw CApexException( ERROR_FATAL, strError, __FILE__, __LINE__);
				}

				break;
		}
	}
}

