#include <stdlib.h>
#include <stdio.h>
#include <err.h>
#include <errno.h>
#include <sys/select.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>

static int _modem_chat(const char *path)
{
    int fd, r;

    if (-1 == (fd = open(path, O_RDWR | O_NOCTTY | O_NONBLOCK))) {
        perror("open");
        return -1;
    }

    if (-1 == (r = tcflush(fd, TCIOFLUSH))) {
        perror("tcflush");
        goto out;
    }

    while (1) {
        char buffer[1024];
        fd_set set;
        int in, out;

        FD_ZERO(&set);
        FD_SET(0, &set);
        FD_SET(fd, &set);

        if (-1 == (r = select(fd + 1, &set, NULL, NULL, NULL))) {
            perror("select");
            goto out;
        }

        if (FD_ISSET(0, &set))
            in = 0, out = fd;
        else if (FD_ISSET(fd, &set))
            in = fd, out = 1;
        else
            continue; // spurious return from select?

        r = (int)read(in, buffer, sizeof(buffer));

        if (-1 == r) {
            if (EAGAIN == errno)
                continue;

            perror("read");
            goto out;
        }
        if (0 == r)
            goto out; // eof

        write(out, buffer, r); // might not complete, don't care
    }

out:
    close(fd);
    return r;
}

int main(int argc, char *argv[])
{
    if (argc < 2) {
        printf("chat /dev/ttyACM0\n");
        return 0;
    }

    return _modem_chat(argv[1]);
}
